// Define the COVID-19 data
const data = {
  name: 'Global',
  children: [
    {
      name: 'Europe',
      value: 12,
      type: 'blue',
      level: 'aqua',
      children: [
        {
          name: 'Germany',
          value: 9,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 150000',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 1000',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 140000',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'France',
          value: 7,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 120000',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 2000',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 100000',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
    {
      name: 'North America',
      value: 10,
      type: 'blue',
      level: 'aqua',
      children: [
        {
          name: 'USA',
          value: 8,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 500000',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 10000',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 400000',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'Canada',
          value: 2,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 80000',
              value: 2,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 3000',
              value: 2,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 60000',
              value: 2,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
    {
      name: 'Africa',
      value: 8,
      type: 'blue',
      level: 'aqua',
      children: [
        {
          name: 'Nigeria',
          value: 4,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 50000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 1000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 45000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'South Africa',
          value: 4,
          type: 'red',
          level: '#863a9f',
          children: [
            {
              name: 'Cumulative Confirmed Cases - 200000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Cumulative Deceased - 5000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries - 150000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
  ],
};

// Define width, height, and margin of the tree
const margin = {
  top: 10,
  right: 250,
  bottom: 200,
  left: 200,
};
const width = 850 - margin.left - margin.right;
const height = 650 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

// Assign hierarchical structure to classify the data
let nodes = d3.hierarchy(data, (node) => node.children);

// Provide data connection for nodes to their respective position in the tree
nodes = tree(nodes);

// Check if SVG already exists
const svgExists = d3.select('svg').empty();

// Add an svg element if it doesn't exist
if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom);
  const g = svg
    .append('g') // appends group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  // Define links between nodes
  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => d.data.level)
    .attr('d', (d) => {
      return (
        'M' +
        d.y +
        ',' +
        d.x +
        'C' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.x +
        ' ' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.parent.x +
        ' ' +
        d.parent.y +
        ',' +
        d.parent.x
      );
    });

  // Declare a group for each node present in the tree
  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  // Define shape of the node
  node
    .append('circle')
    .attr('r', (d) => d.data.value)
    .style('stroke', (d) => d.data.type)
    .style('fill', (d) => d.data.level);

  // Add a new node to the list
  node
    .append('text')
    .attr('dy', '.60em')
    .attr('x', (d) =>
      d.children
        ? (d.data.value + 5) * -1
        : d.data.value + 5,
    )
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -(d.data.value + 5) : d,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name);
}
